--- Variables ---

local constMachineName = "metallurgy-mold-machine"
local constIsInDebug = true


--- Functions ---

local Init = function()
	if not KT_data then 
		global.KT_data = 
		{
			machines ={},
			tick_check = {}
		} 
	end
end

local Update = function(data)
	Init()
end

local eventRemoved = function(event)
	local ent = event.entity
	if not (ent and ent.valid) then return end
	if ent.name == constMachineName then
		global.KT_data.machines[ent.unit_number].drainer.destroy()
		global.KT_data.machines[ent.unit_number] = nil
	end
end

local findTick = function(entity)

	--Credits : https://github.com/Klonan/Total_Automization/blob/master/script/unit_deployment.lua#L1
	
	if not (entity and entity.valid) then return end
	--log(serpent.block(global.KT_data.machines[entity.unit_number].drainer.is_connected_to_electric_network()))
	local recipe = entity.get_recipe()
	local check_tick = game.tick + 30 -- 1 sec
	local fuel = entity.get_fuel_inventory()[1]
	local Drainer = global.KT_data.machines[entity.unit_number].drainer
    if not recipe then
		--No recipe, so lets check this guy again in some ticks			
		global.KT_data.tick_check[check_tick] = global.KT_data.tick_check[check_tick] or {}
		global.KT_data.tick_check[check_tick][entity.unit_number] = entity
		entity.burner.remaining_burning_fuel = 0
		Drainer.get_inventory(defines.inventory.assembling_machine_input).clear()
		Drainer.get_inventory(defines.inventory.assembling_machine_output).clear()
		Drainer.set_recipe(nil)
		entity.active = false
		return
	end
	
	local invAs = entity.get_inventory(defines.inventory.assembling_machine_modules)
	local invDr = Drainer.get_inventory(defines.inventory.assembling_machine_modules)
	
	
	
	if not invAs.is_empty() then
		for i = 1, #invAs do
			invDr[i].set_stack(invAs[i]);
		end
		--log(serpent.block(invAs.get_contents()))
		--log(serpent.block(invDr.get_contents()))
	elseif not invDr.is_empty() then
		invDr.clear()
	end
	
	
	
	
	
	log(serpent.block(Drainer.status))

	if Drainer.status ~= defines.entity_status.low_power and Drainer.status ~= defines.entity_status.no_power then
		local fluids = entity.fluidbox
		
		if not entity.get_fuel_inventory().is_empty() then 
			
			local a, b = string.match(fuel.name, "(.*)%-(.*)")
			local c, d = string.match(recipe.name, "(.*)%-(.*)")
			log(serpent.block(fuel.name.." --> "..a.."  "..b)) 
			log(serpent.block(recipe.name.." --> "..c.."  "..d)) 
			--log(serpent.block(b.." vs "..d)) 
			if b==d then
				entity.active = true
			else
				rendering.draw_text{text="No reicipe with "..entity.fluidbox[1].name.." using the mold "..fuel.name, surface = "nauvis", target = entity,alignment ="center",color = {r=1,g=0,b=0},time_to_live = 61,target_offset = {0,-0.7}}
				entity.active = false
			end
		end
		if entity.burner.remaining_burning_fuel > 0 then
			entity.active = true
			Drainer.active = true
		end
	else 	
		rendering.draw_text{text="Not enough power supplied", surface = "nauvis", target = entity,alignment ="center",color = {r=1,g=0,b=0},time_to_live = 61,target_offset = {0,-0.7}}	
		entity.active = false
		--Drainer.active = false
	end
	--log(serpent.block(entity.is_crafting())) 
	

	if entity.is_crafting() and entity.status == defines.entity_status.working then
		
		Drainer.set_recipe("mold-crafting-invisble")
		--log(serpent.block(Drainer.get_recipe().ingredients[1].name)) 
		if Drainer.get_inventory(defines.inventory.assembling_machine_input).is_empty() then
			Drainer.get_inventory(defines.inventory.assembling_machine_input).insert({name = Drainer.get_recipe().ingredients[1].name,count = 1})
		end
		if not Drainer.get_inventory(defines.inventory.assembling_machine_output).is_empty() then
			
		end
		
		if entity.status == defines.entity_status.item_production_overload then
			Drainer.active = false
		else
			Drainer.active = true
		end
	else	
		Drainer.get_inventory(defines.inventory.assembling_machine_input).clear()
		Drainer.get_inventory(defines.inventory.assembling_machine_output).clear()
		Drainer.set_recipe(nil)
	end
	
	global.KT_data.tick_check[check_tick] = global.KT_data.tick_check[check_tick] or {}
	global.KT_data.tick_check[check_tick][entity.unit_number] = entity
end

local eventBuilt = function(event)
	local ent = event.created_entity
	--log(serpent.block(ent.unit_number.." Was built)"))
	if not (ent and ent.valid) then return end
    if ent.name == constMachineName then
		local Pos = ent.position
		local Name = ent.name
		local Rot = ent.direction
		local Force = ent.force
		local Drainer = game.surfaces.nauvis.create_entity({name = "metallurgy-mold-machine-drainer", position = {Pos.x,Pos.y+5} , direction = Rot, force = Force})
		global.KT_data.machines[ent.unit_number] = {entity = ent,mold = "none", drainer = Drainer}
		--log(serpent.block("Placed "..Drainer.name.." under "..ent.name))
		ent.active = false
		findTick(ent)
    end
	
end

script.on_init(Init)
script.on_configuration_changed(Update)
script.on_event(defines.events.on_built_entity, eventBuilt)
script.on_event(defines.events.on_robot_built_entity, eventBuilt)
script.on_event(defines.events.on_player_mined_entity, eventRemoved)
script.on_event(defines.events.on_robot_mined_entity, eventRemoved)
script.on_event(defines.events.on_entity_died, eventRemoved)

script.on_event(defines.events.on_tick, function(event) 
	local entities = global.KT_data.tick_check[event.tick]
	if not entities then return end
	for unit_number, entity in pairs (entities) do
		findTick(entity)
	end
	global.KT_data.tick_check[event.tick] = nil
end)